@@ -10,14 +10,17 @@ from numba import jit
 from scipy.special import (gamma   as gamma_fn,
                            gammaln as gammaln_fn)
 from .algos import nCk
-from .wavelets import _xifn
-from .configs import gdefaults
+from .wavelets import _xifn, _process_params_dtype
+from .configs import gdefaults, USE_GPU, IS_PARALLEL
+from .utils.backend import torch
+from .utils import backend as S
 
 pi = np.pi
 
 
 #### Base wavelets (`K=1`) ###################################################
-def gmw(gamma=None, beta=None, norm=None, order=None, centered_scale=None):
+def gmw(gamma=None, beta=None, norm=None, order=None, centered_scale=None,
+        dtype=None):
     """Generalized Morse Wavelets. Returns function which computes GMW in the
     frequency domain.
 
@@ -60,6 +63,9 @@ def gmw(gamma=None, beta=None, norm=None, order=None, centered_scale=None):
 
             False by default for consistency with other `ssqueezepy` wavelets.
 
+        dtype: str / type (np.dtype) / None
+            See `help(wavelets.Wavelet)`.
+
     # Returns
         psihfn: function
             Function that computes GMWs, taking `w` (radian frequency)
@@ -111,8 +117,11 @@ def gmw(gamma=None, beta=None, norm=None, order=None, centered_scale=None):
     """
     _check_args(gamma=gamma, beta=beta, norm=norm, order=order)
     kw = gdefaults('_gmw.gmw', gamma=gamma, beta=beta, norm=norm, order=order,
-                   centered_scale=centered_scale, as_dict=True)
+                   centered_scale=centered_scale, dtype=dtype, as_dict=True)
     norm, k = kw.pop('norm'), kw.pop('order')
+    if norm == 'energy'and dtype in ('float32', np.float32):
+        raise ValueError("`norm='energy'` w/ `dtype='float32'` is unsupported; "
+                         "use 'float64' instead.")
 
     l1_fn, l2_fn = ((gmw_l1, gmw_l2) if k == 0 else
                     (gmw_l1_k, gmw_l2_k))
@@ -123,7 +132,7 @@ def gmw(gamma=None, beta=None, norm=None, order=None, centered_scale=None):
 
 
 def compute_gmw(N, scale, gamma=3, beta=60, time=False, norm='bandpass',
-                order=0, centered_scale=False, norm_scale=True):
+                order=0, centered_scale=False, norm_scale=True, dtype=None):
     """Evaluates GMWs, returning as arrays. See `help(_gmw.gmw)` for full docs.
 
     # Arguments
@@ -153,7 +162,7 @@ def compute_gmw(N, scale, gamma=3, beta=60, time=False, norm='bandpass',
             Time-domain wavelet, returned if `time=True`.
     """
     _check_args(gamma=gamma, beta=beta, norm=norm, scale=scale)
-    gmw_fn = gmw(gamma, beta, norm, order, centered_scale)
+    gmw_fn = gmw(gamma, beta, norm, order, centered_scale, dtype)
 
     w = _xifn(scale, N)
     X = np.zeros(N)
@@ -175,25 +184,48 @@ def compute_gmw(N, scale, gamma=3, beta=60, time=False, norm='bandpass',
     return (X, x) if time else X
 
 
-def gmw_l1(gamma=3., beta=60., centered_scale=False):
+def gmw_l1(gamma=3., beta=60., centered_scale=False, dtype='float64'):
     """Generalized Morse Wavelets, first order, L1(bandpass)-normalized.
     See `help(_gmw.gmw)`.
     """
     _check_args(gamma=gamma, beta=beta, allow_zerobeta=False)
     wc = morsefreq(gamma, beta)
+
+    wcl = np.log(wc)
+    gamma, beta, wc, wcl = _process_params_dtype(gamma, beta, wc, wcl, dtype=dtype)
+
+    fn = _gmw_l1_gpu if USE_GPU() else (_gmw_l1_par if IS_PARALLEL() else _gmw_l1)
     if centered_scale:
-        return lambda w: _gmw_l1(np.atleast_1d(w * wc), gamma, beta, wc, w < 0)
+        return lambda w: fn(S.atleast_1d(w * wc, dtype), gamma, beta, wc, wcl)
     else:
-        return lambda w: _gmw_l1(np.atleast_1d(w), gamma, beta, wc, w < 0)
+        return lambda w: fn(S.atleast_1d(w, dtype), gamma, beta, wc, wcl)
 
 @jit(nopython=True, cache=True)
-def _gmw_l1(w, gamma, beta, wc, w_negs):
-    w *= ~w_negs  # zero negative `w` to avoid nans
-    return 2 * np.exp(- beta * np.log(wc) + wc**gamma
-                      + beta * np.log(w)  - w**gamma) * (~w_negs)
+def _gmw_l1(w, gamma, beta, wc, wcl):
+    # NOTE: numba.jit, unlike numpy & torch, will promote to float64 with
+    # array float32 and scalar float64
+    w_nonneg = (w >= 0)
+    w *= w_nonneg  # zero negative `w` to avoid nans
+    return 2 * np.exp(- beta * wcl + wc**gamma
+                      + beta * np.log(w) - w**gamma) * w_nonneg
+
+@jit(nopython=True, cache=True, parallel=True)
+def _gmw_l1_par(w, gamma, beta, wc, wcl):
+    # NOTE: numba.jit, unlike numpy & torch, will promote to float64 with
+    # array float32 and scalar float64
+    w_nonneg = (w >= 0)
+    w *= w_nonneg  # zero negative `w` to avoid nans
+    return 2 * np.exp(- beta * wcl + wc**gamma
+                      + beta * np.log(w) - w**gamma) * w_nonneg
+
+def _gmw_l1_gpu(w, gamma, beta, wc, wcl):
+    w_nonneg = (w >= 0)
+    w *= w_nonneg
+    return 2 * torch.exp(- beta * wcl + wc**gamma
+                         + beta * torch.log(w) - w**gamma) * w_nonneg
 
 
-def gmw_l2(gamma=3., beta=60., centered_scale=False):
+def gmw_l2(gamma=3., beta=60., centered_scale=False, dtype='float64'):
     """Generalized Morse Wavelets, first order, L2(energy)-normalized.
     See `help(_gmw.gmw)`.
     """
@@ -201,77 +233,137 @@ def gmw_l2(gamma=3., beta=60., centered_scale=False):
     wc = morsefreq(gamma, beta)
     r = (2*beta + 1) / gamma
     rgamma = gamma_fn(r)
+    (gamma, beta, wc, r, rgamma
+     ) = _process_params_dtype(gamma, beta, wc, r, rgamma, dtype=dtype)
 
+    fn = _gmw_l2_gpu if USE_GPU() else (_gmw_l2_par if IS_PARALLEL() else _gmw_l2)
     if centered_scale:
-        return lambda w: _gmw_l2(np.atleast_1d(w * wc), gamma, beta, wc,
-                                 r, rgamma, w < 0)
+        return lambda w: fn(S.atleast_1d(w * wc, dtype), gamma, beta, wc,
+                            r, rgamma)
     else:
-        return lambda w: _gmw_l2(np.atleast_1d(w), gamma, beta, wc,
-                                 r, rgamma, w < 0)
+        return lambda w: fn(S.atleast_1d(w, dtype), gamma, beta, wc, r, rgamma)
 
 @jit(nopython=True, cache=True)
-def _gmw_l2(w, gamma, beta, wc, r, rgamma, w_negs):
-    w *= ~w_negs  # zero negative `w` to avoid nans
+def _gmw_l2(w, gamma, beta, wc, r, rgamma):
+    w_nonneg = (w >= 0)
+    w *= w_nonneg  # zero negative `w` to avoid nans
     return np.sqrt(2.*pi * gamma * 2.**r / rgamma
-                   ) * w**beta * np.exp(-w**gamma) * (~w_negs)
+                   ) * w**beta * np.exp(-w**gamma) * w_nonneg
+
+@jit(nopython=True, cache=True, parallel=True)
+def _gmw_l2_par(w, gamma, beta, wc, r, rgamma):
+    w_nonneg = (w >= 0)
+    w *= w_nonneg  # zero negative `w` to avoid nans
+    return np.sqrt(2.*pi * gamma * 2.**r / rgamma
+                   ) * w**beta * np.exp(-w**gamma) * w_nonneg
+
+def _gmw_l2_gpu(w, gamma, beta, wc, r, rgamma):
+    w_nonneg = (w >= 0)
+    w *= w_nonneg  # zero negative `w` to avoid nans
+    return torch.sqrt(2.*pi * gamma * 2.**r / rgamma
+                      ) * w**beta * torch.exp(-w**gamma) * w_nonneg
 
 
-def gmw_l1_k(gamma=3., beta=60., k=1, centered_scale=False):
+def gmw_l1_k(gamma=3., beta=60., k=1, centered_scale=False, dtype='float64'):
     """Generalized Morse Wavelets, `k`-th order, L1(bandpass)-normalized.
     See `help(_gmw.gmw)`.
     """
     _check_args(gamma=gamma, beta=beta, allow_zerobeta=False)
 
     wc = morsefreq(gamma, beta)
-    k_consts = _gmw_k_constants(gamma, beta, k, norm='bandpass')
+    k_consts = _gmw_k_constants(gamma, beta, k, norm='bandpass', dtype=dtype)
+    gamma, beta, wc = _process_params_dtype(gamma, beta, wc, dtype=dtype)
 
+    fn = (_gmw_l1_k_gpu if USE_GPU() else
+          (_gmw_l1_k_par if IS_PARALLEL() else _gmw_l1_k))
     if centered_scale:
-        return lambda w: _gmw_l1_k(np.atleast_1d(w * wc), gamma, beta, wc, w < 0,
-                                   k_consts)
+        return lambda w: fn(S.atleast_1d(w * wc, dtype), gamma, beta, wc,
+                            k_consts)
     else:
-        return lambda w: _gmw_l1_k(np.atleast_1d(w), gamma, beta, wc, w < 0,
-                                   k_consts)
+        return lambda w: fn(S.atleast_1d(w, dtype), gamma, beta, wc, k_consts)
 
 @jit(nopython=True, cache=True)
-def _gmw_l1_k(w, gamma, beta, wc, w_negs, k_consts):
-    w *= ~w_negs  # zero negative `w` to avoid nans
+def _gmw_l1_k(w, gamma, beta, wc, k_consts):
+    w_nonneg = (w >= 0)
+    w *= w_nonneg  # zero negative `w` to avoid nans
 
-    C = np.zeros(w.shape)
+    C = np.zeros(w.shape, dtype=w.dtype)
     for m in range(len(k_consts)):
         C += k_consts[m] * (2*w**gamma)**m
-
     return C * np.exp(- beta * np.log(wc) + wc**gamma
-                      + beta * np.log(w)  - w**gamma) * (~w_negs)
+                      + beta * np.log(w)  - w**gamma) * w_nonneg
+
+@jit(nopython=True, cache=True, parallel=True)
+def _gmw_l1_k_par(w, gamma, beta, wc, k_consts):
+    w_nonneg = (w >= 0)
+    w *= w_nonneg  # zero negative `w` to avoid nans
+
+    C = np.zeros(w.shape, dtype=w.dtype)
+    for m in range(len(k_consts)):
+        C += k_consts[m] * (2*w**gamma)**m
+    return C * np.exp(- beta * np.log(wc) + wc**gamma
+                      + beta * np.log(w)  - w**gamma) * w_nonneg
+
+def _gmw_l1_k_gpu(w, gamma, beta, wc, k_consts):
+    w_nonneg = (w >= 0)
+    w *= w_nonneg  # zero negative `w` to avoid nans
+
+    C = w.new_zeros(w.shape)
+    for m in range(len(k_consts)):
+        C += k_consts[m] * (2*w**gamma)**m
+    return C * torch.exp(- beta * torch.log(wc) + wc**gamma
+                         + beta * torch.log(w)  - w**gamma) * w_nonneg
 
 
-def gmw_l2_k(gamma=3., beta=60., k=1, centered_scale=False):
+def gmw_l2_k(gamma=3., beta=60., k=1, centered_scale=False, dtype='float64'):
     """Generalized Morse Wavelets, `k`-th order, L2(energy)-normalized.
     See `help(_gmw.gmw)`.
     """
     _check_args(gamma=gamma, beta=beta, allow_zerobeta=False)
 
     wc = morsefreq(gamma, beta)
-    k_consts = _gmw_k_constants(gamma, beta, k, norm='energy')
+    k_consts = _gmw_k_constants(gamma, beta, k, norm='energy', dtype=dtype)
+    gamma, beta, wc = _process_params_dtype(gamma, beta, wc, dtype=dtype)
 
+    fn = (_gmw_l2_k_gpu if USE_GPU() else
+          (_gmw_l2_k_par if IS_PARALLEL() else _gmw_l2_k))
     if centered_scale:
-        return lambda w: _gmw_l2_k(np.atleast_1d(w * wc), gamma, beta, wc, w < 0,
-                                   k_consts)
+        return lambda w: fn(S.atleast_1d(w * wc, dtype), gamma, beta, wc,
+                            k_consts)
     else:
-        return lambda w: _gmw_l2_k(np.atleast_1d(w), gamma, beta, wc, w < 0,
-                                   k_consts)
+        return lambda w: fn(S.atleast_1d(w, dtype), gamma, beta, wc, k_consts)
 
 @jit(nopython=True, cache=True)
-def _gmw_l2_k(w, gamma, beta, wc, w_negs, k_consts):
-    w *= ~w_negs  # zero negative `w` to avoid nans
+def _gmw_l2_k(w, gamma, beta, wc, k_consts):
+    w_nonneg = (w >= 0)
+    w *= w_nonneg  # zero negative `w` to avoid nans
 
-    C = np.zeros(w.shape)
+    C = np.zeros(w.shape, dtype=w.dtype)
     for m in range(len(k_consts)):
         C += k_consts[m] * (2*w**gamma)**m
+    return C * np.exp(beta * np.log(w) - w**gamma) * w_nonneg
 
-    return C * np.exp(beta * np.log(w) - w**gamma) * (~w_negs)
+@jit(nopython=True, cache=True, parallel=True)
+def _gmw_l2_k_par(w, gamma, beta, wc, k_consts):
+    w_nonneg = (w >= 0)
+    w *= w_nonneg  # zero negative `w` to avoid nans
+
+    C = np.zeros(w.shape, dtype=w.dtype)
+    for m in range(len(k_consts)):
+        C += k_consts[m] * (2*w**gamma)**m
+    return C * np.exp(beta * np.log(w) - w**gamma) * w_nonneg
+
+def _gmw_l2_k_gpu(w, gamma, beta, wc, k_consts):
+    w_nonneg = (w >= 0)
+    w *= w_nonneg  # zero negative `w` to avoid nans
+
+    C = w.new_zeros(w.shape)
+    for m in range(len(k_consts)):
+        C += k_consts[m] * (2*w**gamma)**m
+    return C * torch.exp(beta * torch.log(w) - w**gamma) * w_nonneg
 
 
-def _gmw_k_constants(gamma, beta, k, norm='bandpass'):
+def _gmw_k_constants(gamma, beta, k, norm='bandpass', dtype='float64'):
     """Laguerre polynomial constants & `coeff` term.
 
     Higher-order GMWs are coded such that constants are pre-computed and reused
@@ -289,7 +381,7 @@ def _gmw_k_constants(gamma, beta, k, norm='bandpass'):
                         np.exp(gammaln_fn(k + 1) - gammaln_fn(k + r)))
 
     # compute Laguerre polynomial constants
-    L_consts = np.zeros(k + 1)
+    L_consts = np.zeros(k + 1, dtype=dtype)
     for m in range(k + 1):
         fact = np.exp(gammaln_fn(k + c + 1) - gammaln_fn(c + m + 1) -
                       gammaln_fn(k - m + 1))
@@ -298,8 +390,10 @@ def _gmw_k_constants(gamma, beta, k, norm='bandpass'):
     k_consts = L_consts * coeff
     if norm == 'bandpass':
         k_consts *= 2
+    k_consts = k_consts.astype(dtype)
     return k_consts
 
+
 #### General order wavelets (any `K`) ########################################
 def morsewave(N, freqs, gamma=3, beta=60, K=1, norm='bandpass'):
     """Generalized Morse wavelets of Olhede and Walden (2002).
